@@ -58,8 +58,6 @@ import java.util.Comparator;
 import java.util.List;
 import java.util.Locale;
 import java.util.concurrent.ExecutionException;
-import java.util.concurrent.Executor;
-import java.util.concurrent.Future;
 import java.util.concurrent.TimeUnit;
 import com.openexchange.exception.Category;
 import com.openexchange.exception.OXException;
@@ -92,8 +90,8 @@ import com.openexchange.threadpool.ThreadPools;
 import com.openexchange.unifiedinbox.converters.UnifiedInboxFolderConverter;
 import com.openexchange.unifiedinbox.services.Services;
 import com.openexchange.unifiedinbox.utility.LoggingCallable;
-import com.openexchange.unifiedinbox.utility.TrackingCompletionService;
 import com.openexchange.unifiedinbox.utility.UnifiedInboxCompletionService;
+import com.openexchange.unifiedinbox.utility.UnifiedInboxCompletionService.Result;
 import com.openexchange.unifiedinbox.utility.UnifiedInboxUtility;
 import com.openexchange.user.UserService;
 
@@ -154,10 +152,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             return;
         }
         if (UnifiedInboxAccess.KNOWN_FOLDERS.contains(fullName)) {
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<Void> completionService = new UnifiedInboxCompletionService<Void>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<Void> completionService = new UnifiedInboxCompletionService<Void>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<Void>(session) {
 
@@ -173,11 +170,14 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                             if (fn == null) {
                                 return null;
                             }
-                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-                            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                                ((IMailFolderStorageEnhanced) messageStorage).expungeFolder(fn, hardDelete);
+
+                            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+                            if (null != storageEnhanced) {
+                                storageEnhanced.expungeFolder(fn, hardDelete);
                                 return null;
                             }
+
+                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
                             final MailField[] fields = new MailField[] { MailField.ID };
                             final FlagTerm term = new FlagTerm(MailMessage.FLAG_DELETED, true);
                             final MailMessage[] messages = messageStorage.searchMessages(fn, IndexRange.NULL, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields);
@@ -231,14 +231,17 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
             // Get account's messages
-            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
             final String fn = fa.getFullname();
-            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                ((IMailFolderStorageEnhanced) messageStorage).expungeFolder(fn, hardDelete);
+
+            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+            if (null != storageEnhanced) {
+                storageEnhanced.expungeFolder(fn, hardDelete);
                 return;
             }
+
             final MailField[] fields = new MailField[] { MailField.ID };
             final FlagTerm term = new FlagTerm(MailMessage.FLAG_DELETED, true);
+            IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
             final MailMessage[] messages = messageStorage.searchMessages(fn, IndexRange.NULL, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields);
             final List<String> mailIds = new ArrayList<String>(messages.length);
             for (int i = 0; i < messages.length; i++) {
@@ -269,10 +272,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             return new int[] {0,0};
         }
         if (UnifiedInboxAccess.KNOWN_FOLDERS.contains(fullName)) {
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<int[]> completionService = new UnifiedInboxCompletionService<int[]>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<int[]> completionService = new UnifiedInboxCompletionService<int[]>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<int[]>(session) {
 
@@ -289,13 +291,17 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                                 return new int[] {0,0};
                             }
                             IMailFolderStorage folderStorage = mailAccess.getFolderStorage();
-                            if (folderStorage instanceof IMailFolderStorageEnhanced2) {
-                                return ((IMailFolderStorageEnhanced2) folderStorage).getTotalAndUnreadCounter(fn);
+
+                            IMailFolderStorageEnhanced2 storageEnhanced2 = folderStorage.supports(IMailFolderStorageEnhanced2.class);
+                            if (null != storageEnhanced2) {
+                                return storageEnhanced2.getTotalAndUnreadCounter(fn);
                             }
-                            if (folderStorage instanceof IMailFolderStorageEnhanced) {
-                                final IMailFolderStorageEnhanced enhanced = (IMailFolderStorageEnhanced) folderStorage;
-                                return new int[] { enhanced.getTotalCounter(fn), enhanced.getUnreadCounter(fn) };
+
+                            IMailFolderStorageEnhanced storageEnhanced = folderStorage.supports(IMailFolderStorageEnhanced.class);
+                            if (null != storageEnhanced) {
+                                return new int[] { storageEnhanced.getTotalCounter(fn), storageEnhanced.getUnreadCounter(fn) };
                             }
+
                             final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
                             final MailField[] fields = new MailField[] { MailField.ID };
                             final int count = messageStorage.searchMessages(fn, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, fields).length;
@@ -303,6 +309,8 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                             final int unreadCount = unreadMessages.length;
                             return new int[] { count, unreadCount };
                         } catch (final OXException e) {
+                            e.setCategory(Category.CATEGORY_WARNING);
+                            access.addWarnings(Collections.singleton(e));
                             getLogger().debug("", e);
                             return new int[] {0,0};
                         } finally {
@@ -341,15 +349,19 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             mailAccess.connect();
             // Get account's messages
             IMailFolderStorage folderStorage = mailAccess.getFolderStorage();
-            if (folderStorage instanceof IMailFolderStorageEnhanced2) {
-                return ((IMailFolderStorageEnhanced2) folderStorage).getTotalAndUnreadCounter(fa.getFullname());
+
+            IMailFolderStorageEnhanced2 storageEnhanced2 = folderStorage.supports(IMailFolderStorageEnhanced2.class);
+            if (null != storageEnhanced2) {
+                return storageEnhanced2.getTotalAndUnreadCounter(fa.getFullname());
             }
-            if (folderStorage instanceof IMailFolderStorageEnhanced) {
-                final IMailFolderStorageEnhanced enhanced = (IMailFolderStorageEnhanced) folderStorage;
-                final int count = enhanced.getTotalCounter(fa.getFullname());
-                final int unreadCount = enhanced.getUnreadCounter(fa.getFullname());
+
+            IMailFolderStorageEnhanced storageEnhanced = folderStorage.supports(IMailFolderStorageEnhanced.class);
+            if (null != storageEnhanced) {
+                final int count = storageEnhanced.getTotalCounter(fa.getFullname());
+                final int unreadCount = storageEnhanced.getUnreadCounter(fa.getFullname());
                 return new int[] { count, unreadCount };
             }
+
             final MailField[] fields = new MailField[] { MailField.ID };
             final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
             final int count = messageStorage.searchMessages(fa.getFullname(), null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, fields).length;
@@ -367,10 +379,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             return 0;
         }
         if (UnifiedInboxAccess.KNOWN_FOLDERS.contains(fullName)) {
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<Integer>(session) {
 
@@ -386,10 +397,13 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                             if (fn == null) {
                                 return Integer.valueOf(0);
                             }
-                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-                            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                                return Integer.valueOf(((IMailFolderStorageEnhanced) messageStorage).getTotalCounter(fn));
+
+                            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+                            if (null != storageEnhanced) {
+                                return Integer.valueOf(storageEnhanced.getTotalCounter(fn));
                             }
+
+                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
                             final MailField[] fields = new MailField[] { MailField.ID };
                             final int count = messageStorage.searchMessages(fn, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, fields).length;
                             return Integer.valueOf(count);
@@ -425,12 +439,14 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
             // Get account's messages
-            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                return ((IMailFolderStorageEnhanced) messageStorage).getTotalCounter(fa.getFullname());
+
+            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+            if (null != storageEnhanced) {
+                return storageEnhanced.getTotalCounter(fa.getFullname());
             }
+
             final MailField[] fields = new MailField[] { MailField.ID };
-            final int count = messageStorage.searchMessages(fa.getFullname(), null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, fields).length;
+            final int count = mailAccess.getMessageStorage().searchMessages(fa.getFullname(), null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, null, fields).length;
             return count;
         } finally {
             closeSafe(mailAccess);
@@ -443,10 +459,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             return 0;
         }
         if (UnifiedInboxAccess.KNOWN_FOLDERS.contains(fullName)) {
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<Integer>(session) {
 
@@ -462,13 +477,15 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                             if (fn == null) {
                                 return Integer.valueOf(0);
                             }
-                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-                            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                                return Integer.valueOf(((IMailFolderStorageEnhanced) messageStorage).getNewCounter(fn));
+
+                            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+                            if (null != storageEnhanced) {
+                                return Integer.valueOf(storageEnhanced.getNewCounter(fn));
                             }
+
                             final MailField[] fields = new MailField[] { MailField.ID };
                             final SearchTerm<?> term = new FlagTerm(MailMessage.FLAG_RECENT, true);
-                            final int count = messageStorage.searchMessages(fn, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields).length;
+                            final int count = mailAccess.getMessageStorage().searchMessages(fn, null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields).length;
                             return Integer.valueOf(count);
                         } catch (final OXException e) {
                             getLogger().debug("", e);
@@ -503,13 +520,15 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
             // Get account's messages
-            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                return ((IMailFolderStorageEnhanced) messageStorage).getUnreadCounter(fa.getFullname());
+
+            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+            if (null != storageEnhanced) {
+                return storageEnhanced.getUnreadCounter(fa.getFullname());
             }
+
             final MailField[] fields = new MailField[] { MailField.ID };
             final SearchTerm<?> term = new FlagTerm(MailMessage.FLAG_RECENT, true);
-            final int count = messageStorage.searchMessages(fa.getFullname(), null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields).length;
+            final int count = mailAccess.getMessageStorage().searchMessages(fa.getFullname(), null, MailSortField.RECEIVED_DATE, OrderDirection.ASC, term, fields).length;
             return count;
         } finally {
             closeSafe(mailAccess);
@@ -522,10 +541,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             return 0;
         }
         if (UnifiedInboxAccess.KNOWN_FOLDERS.contains(fullName)) {
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<Integer> completionService = new UnifiedInboxCompletionService<Integer>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<Integer>(session) {
 
@@ -541,13 +559,14 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
                             if (fn == null) {
                                 return Integer.valueOf(0);
                             }
-                            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-                            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                                return Integer.valueOf(((IMailFolderStorageEnhanced) messageStorage).getUnreadCounter(fn));
+
+                            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+                            if (null != storageEnhanced) {
+                                return Integer.valueOf(storageEnhanced.getUnreadCounter(fn));
                             }
+
                             final MailField[] fields = new MailField[] { MailField.ID };
-                            final MailMessage[] unreadMessages =
-                                messageStorage.getUnreadMessages(fn, MailSortField.RECEIVED_DATE, OrderDirection.ASC, fields, -1);
+                            final MailMessage[] unreadMessages = mailAccess.getMessageStorage().getUnreadMessages(fn, MailSortField.RECEIVED_DATE, OrderDirection.ASC, fields, -1);
                             return Integer.valueOf(unreadMessages.length);
                         } catch (final OXException e) {
                             getLogger().debug("", e);
@@ -582,13 +601,14 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             mailAccess = MailAccess.getInstance(session, accountId);
             mailAccess.connect();
             // Get account's messages
-            final IMailMessageStorage messageStorage = mailAccess.getMessageStorage();
-            if (messageStorage instanceof IMailFolderStorageEnhanced) {
-                return ((IMailFolderStorageEnhanced) messageStorage).getUnreadCounter(fa.getFullname());
+
+            IMailFolderStorageEnhanced storageEnhanced = mailAccess.getFolderStorage().supports(IMailFolderStorageEnhanced.class);
+            if (null != storageEnhanced) {
+                return storageEnhanced.getUnreadCounter(fa.getFullname());
             }
+
             final MailField[] fields = new MailField[] { MailField.ID };
-            final MailMessage[] unreadMessages =
-                messageStorage.getUnreadMessages(fa.getFullname(), MailSortField.RECEIVED_DATE, OrderDirection.ASC, fields, -1);
+            final MailMessage[] unreadMessages = mailAccess.getMessageStorage().getUnreadMessages(fa.getFullname(), MailSortField.RECEIVED_DATE, OrderDirection.ASC, fields, -1);
             return unreadMessages.length;
         } finally {
             closeSafe(mailAccess);
@@ -711,9 +731,8 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             }
             accounts = l.toArray(new MailAccount[l.size()]);
         }
-        final int nAccounts = accounts.length;
-        final Executor executor = ThreadPools.getThreadPool().getExecutor();
-        final TrackingCompletionService<int[][]> completionService = new UnifiedInboxCompletionService<int[][]>(executor);
+        int nAccounts = accounts.length;
+        UnifiedInboxCompletionService<int[][]> completionService = new UnifiedInboxCompletionService<int[][]>(ThreadPools.getThreadPool());
         // Create a task for each account
         for (int i = 0; i < nAccounts; i++) {
             final int accountId = accounts[i].getId();
@@ -752,9 +771,8 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
     }
 
     private MailFolder[] getRootSubfoldersByFolder() throws OXException {
-        final MailFolder[] retval = new MailFolder[5];
-        final Executor executor = ThreadPools.getThreadPool().getExecutor();
-        final TrackingCompletionService<Retval> completionService = new UnifiedInboxCompletionService<Retval>(executor);
+        MailFolder[] retval = new MailFolder[5];
+        UnifiedInboxCompletionService<Retval> completionService = new UnifiedInboxCompletionService<Retval>(ThreadPools.getThreadPool());
         // Init names
         final String[][] names = new String[5][];
         names[0] = new String[] { UnifiedInboxAccess.INBOX, getLocalizedName(UnifiedInboxAccess.INBOX) };
@@ -783,7 +801,7 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
             int completed = 0;
             while (completed < retval.length) {
                 // No timeout
-                final Future<Retval> future = completionService.poll(UnifiedInboxUtility.getMaxRunningMillis(), TimeUnit.MILLISECONDS);
+                Result<Retval> future = completionService.poll(UnifiedInboxUtility.getMaxRunningMillis(), TimeUnit.MILLISECONDS);
                 completed++;
                 if (null != future) {
                     final Retval r = future.get();
@@ -818,9 +836,8 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
         }
         final Session s = session;
         final int unifiedInboxAccountId = unifiedInboxId;
-        final int length = accounts.length;
-        final Executor executor = ThreadPools.getThreadPool().getExecutor();
-        final TrackingCompletionService<MailFolder> completionService = new UnifiedInboxCompletionService<MailFolder>(executor);
+        int length = accounts.length;
+        UnifiedInboxCompletionService<MailFolder> completionService = new UnifiedInboxCompletionService<MailFolder>(ThreadPools.getThreadPool());
         for (final MailAccount mailAccount : accounts) {
             completionService.submit(new LoggingCallable<MailFolder>(session) {
 
@@ -927,10 +944,9 @@ public final class UnifiedInboxFolderStorage extends MailFolderStorage implement
              */
 
             // Clear Unified Mail folder
-            final List<MailAccount> accounts = getAccounts();
-            final int length = accounts.size();
-            final Executor executor = ThreadPools.getThreadPool().getExecutor();
-            final TrackingCompletionService<Void> completionService = new UnifiedInboxCompletionService<Void>(executor);
+            List<MailAccount> accounts = getAccounts();
+            int length = accounts.size();
+            UnifiedInboxCompletionService<Void> completionService = new UnifiedInboxCompletionService<Void>(ThreadPools.getThreadPool());
             for (final MailAccount mailAccount : accounts) {
                 completionService.submit(new LoggingCallable<Void>(session) {
 
