@@ -49,8 +49,6 @@
 
 package com.openexchange.unifiedinbox.copy;
 
-import gnu.trove.map.TIntObjectMap;
-import gnu.trove.map.hash.TIntObjectHashMap;
 import java.util.Collection;
 import java.util.concurrent.ExecutionException;
 import com.openexchange.exception.OXException;
@@ -69,6 +67,8 @@ import com.openexchange.threadpool.behavior.CallerRunsBehavior;
 import com.openexchange.unifiedinbox.UnifiedInboxAccess;
 import com.openexchange.unifiedinbox.UnifiedInboxException;
 import com.openexchange.unifiedinbox.utility.UnifiedInboxUtility;
+import gnu.trove.map.TIntObjectMap;
+import gnu.trove.map.hash.TIntObjectHashMap;
 
 /**
  * {@link UnifiedInboxMessageCopier} - Copies messages from/to Unified Mail folders.
@@ -212,7 +212,7 @@ public final class UnifiedInboxMessageCopier {
         /*
          * A copy/move from a specific folder to this account's default folder
          */
-        final String[] retval;
+        String[] retval;
         // Parse source folder
         final FullnameArgument sourceFullnameArgument = UnifiedInboxUtility.parseNestedFullName(sourceFolder);
         MailAccess<?, ?> mailAccess = null;
@@ -229,7 +229,7 @@ public final class UnifiedInboxMessageCopier {
             } else {
                 retval = mailAccess.getMessageStorage().copyMessages(sourceFullname, realDest, mailIds, fast);
             }
-
+            retval = transformMailId2UnifiedMailId(sourceFullnameArgument.getAccountId(), realDest, retval);
         } finally {
             if (null != mailAccess) {
                 mailAccess.close(true);
@@ -238,6 +238,25 @@ public final class UnifiedInboxMessageCopier {
         return retval;
     }
 
+    /**
+     * Transforms an array of normal mail ids to an array of unified mail ids
+     *
+     * @param account The id of the mail account
+     * @param dest The folder fullname
+     * @param mailids An array of normal mail ids
+     * @return An array of unified mail ids
+     */
+    private String[] transformMailId2UnifiedMailId(int account, String dest, String[] mailids) {
+        UnifiedInboxUID helper = new UnifiedInboxUID();
+        String[] result = new String[mailids.length];
+        int x=0;
+        for(String id: mailids) {
+            result[x++] = helper.setUID(account, dest, id).toString();
+        }
+        
+        return result;
+    }
+
     private String[] accountFolder2AccountFolder(final String sourceFolder, final String destFolder, final String[] mailIds, final boolean fast, final boolean move) throws OXException {
         /*
          * A copy/move from a specific folder to an account's specific folder
@@ -257,10 +276,13 @@ public final class UnifiedInboxMessageCopier {
             try {
                 mailAccess = MailAccess.getInstance(session, sourceFullnameArgument.getAccountId());
                 mailAccess.connect();
+                String[] retval;
                 if (move) {
-                    return mailAccess.getMessageStorage().moveMessages(sourceFullname, destFullname, mailIds, fast);
+                    retval = mailAccess.getMessageStorage().moveMessages(sourceFullname, destFullname, mailIds, fast);
+                } else {
+                    retval = mailAccess.getMessageStorage().copyMessages(sourceFullname, destFullname, mailIds, fast);
                 }
-                return mailAccess.getMessageStorage().copyMessages(sourceFullname, destFullname, mailIds, fast);
+                return transformMailId2UnifiedMailId(sourceFullnameArgument.getAccountId(), destFullname, retval);
             } finally {
                 if (null != mailAccess) {
                     mailAccess.close(true);
@@ -291,7 +313,7 @@ public final class UnifiedInboxMessageCopier {
                 sourceMailAccess.close(true);
             }
         }
-        return retval;
+        return transformMailId2UnifiedMailId(destFullnameArgument.getAccountId(), destFullname, retval);
     }
 
     private static void performCallables(final Collection<? extends Task<Object>> callables, final ThreadPoolService threadPoolService) throws OXException {
